@@ -1,45 +1,57 @@
+import os
+import subprocess as sb
 import time
+from pathlib import Path
 
 import click
 import pexpect
-import os
-import subprocess as sb
 from click_default_group import DefaultGroup
-from pathlib import Path
-from prompt_toolkit import prompt
+from prompt_toolkit.completion import WordCompleter
 from prompt_toolkit.history import InMemoryHistory
+from prompt_toolkit.lexers import PygmentsLexer
+from prompt_toolkit.shortcuts import prompt
+from pygments.lexers.gdscript import GDScriptLexer
 
 from .client import client as wsclient
-from .config import GODOT, VI, PORT
+from .constants import GODOT, KEYWORDS, PORT, VI
 
 STDOUT_MARKER_START = "----------------STDOUT-----------------------"
 STDOUT_MARKER_END = "----------------STDOUT END-----------------------"
 
 
-print("Welcome to GDScript REPL. Hit Ctrl+C to exit. If you start having errors type 'clear'")
+def script_dir():
+    return str(Path(__file__).parent.resolve() / Path("gdserver.gd"))
 
-repl_script_path = str(Path(__file__).parent.resolve() / Path("gdserver.gd"))
+repl_script_path = script_dir()
 
 
 @click.group(cls=DefaultGroup, default='repl', default_if_no_args=True)
 def cli():
     pass
 
-@cli.command(help="Launch the godot server and start teh repl")
+
+@cli.command(help="Launch the godot server and starts the repl")
 @click.option("--vi", is_flag=True, default=VI, help="Use vi mode")
 @click.option("--godot", default=GODOT, help="Path to godot executable")
-def repl(vi, godot):
+@click.option("--command", default="", help="Custom command to run the server script with")
+def repl(vi, godot, command):
+    print("Welcome to GDScript REPL. Hit Ctrl+C to exit. If you start having errors type 'clear'")
     if not godot:
         return
-    server = pexpect.spawn(f"{godot} --script {repl_script_path}")
-    server.expect("Godot Engine v.*")
-    time.sleep(1)
+    server = None
+    if command:
+        server = pexpect.spawn(command)
+    else:
+        server = pexpect.spawn(f"{godot} --script {repl_script_path}")
+    server.expect("Gdrepl Listening on .*")
     client = wsclient()
 
     history = InMemoryHistory()
+    completer = WordCompleter(KEYWORDS, WORD=True)
     while True:
         try:
-            cmd = prompt(">>> ", vi_mode=vi, history=history)
+            cmd = prompt(">>> ", vi_mode=vi, history=history,
+                         lexer=PygmentsLexer(GDScriptLexer), completer=completer)
         except (EOFError, KeyboardInterrupt):
             client.close()
             break
@@ -47,10 +59,15 @@ def repl(vi, godot):
         if len(cmd.strip()) == 0:
             continue
 
+        if cmd == "quit":
+            client.send(cmd, False)
+            client.close()
+            break
+
+        history._loaded_strings = list(dict.fromkeys(history._loaded_strings))
         resp = client.send(cmd)
         if resp:
             print(resp)
-        history.append_string(cmd)
 
         try:
             server.expect(STDOUT_MARKER_START, 0.2)
@@ -67,16 +84,22 @@ def repl(vi, godot):
         except pexpect.exceptions.TIMEOUT:
             pass
 
+
 @cli.command(help="Connects to a running godot repl server")
 @click.option("--vi", is_flag=True, default=VI, help="Use vi mode")
 @click.option("--port", default=PORT, help="Port to listen on")
 def client(vi, port):
+    print("Welcome to GDScript REPL. Hit Ctrl+C to exit. If you start having errors type 'clear'")
     print("Not launching server..")
+
+    # TODO avoid repeating this whole code here again
     client = wsclient(port=port)
     history = InMemoryHistory()
+    completer = WordCompleter(KEYWORDS, WORD=True)
     while True:
         try:
-            cmd = prompt(">>> ", vi_mode=VI, history=history)
+            cmd = prompt(">>> ", vi_mode=vi, history=history,
+                         lexer=PygmentsLexer(GDScriptLexer), completer=completer)
         except (EOFError, KeyboardInterrupt):
             client.close()
             break
@@ -84,10 +107,17 @@ def client(vi, port):
         if len(cmd.strip()) == 0:
             continue
 
+        if cmd == "quit":
+            client.send(cmd, False)
+            client.close()
+            break
+
         resp = client.send(cmd)
         if resp:
             print(resp)
-        history.append_string(cmd)
+
+        history._loaded_strings = list(dict.fromkeys(history._loaded_strings))
+
 
 @cli.command(help="Starts the gdscript repl websocket server")
 @click.option("--godot", default=GODOT, help="Path to godot executable")
